/*********************************************************************************
Date: June 2012, updated July 2014
Purpose: compares weak, zero-centred and non-zero-centred priors.  
	All have no prior on the intecept
Note: This syntax should create Table 3 from the paper but needs the rounding 
	macro to do so
Created by: Sheena Sullivan 
*********************************************************************************/

%let workdir= C:\Users\sullivsh\Dropbox\Bayes\SAS; *assign your own directory;
libname bayes "&workdir";
filename fm0 "&workdir\fm0.asc";
libname mymacs "&workdir\macros";
options mstored sasmstore=mymacs;
options nodate nonumber nocenter formdlim= " " ;

/*******************************************************************
Import the data
********************************************************************/;
data fm0;														
infile fm0;
	input death nonwhite teenages nullip gestage isoimm abort hydram dyslab 
		placord nomonit twint ward prerupt malpres;
	dyslab=dyslab/3; 
	Const=1;
	M1=1;
	H=0;
	run;
/*ML estimates on actual data*/;
proc logistic data=fm0 descending;	
	title "Maximum likelihood estimates, death on 14 regressors";
	model death = nonwhite--malpres / clodds=both;
	ods output cloddswald=actualw cloddspl=actualpl; 
	run;

/*******************************************************************
Create the prior data
********************************************************************/;
data prior1; 
	input death nonwhite teenages nullip gestage isoimm abort hydram dyslab placord nomonit twint ward prerupt malpres Const M1 H;
datalines;
4.5	1	0	0	0	0	0	0	0	0	0	0	0	0	0	0	9	-0.693147181
4.5	0	1	0	0	0	0	0	0	0	0	0	0	0	0	0	9	-0.693147181
4.5	0	0	1	0	0	0	0	0	0	0	0	0	0	0	0	9	-0.693147181
4.5	0	0	0	1	0	0	0	0	0	0	0	0	0	0	0	9	-1.386294361
4.5	0	0	0	0	1	0	0	0	0	0	0	0	0	0	0	9	-0.693147181
4.5	0	0	0	0	0	1	0	0	0	0	0	0	0	0	0	9	0
4.5	0	0	0	0	0	0	1	0	0	0	0	0	0	0	0	9	-1.386294361
4.5	0	0	0	0	0	0	0	1	0	0	0	0	0	0	0	9	-0.693147181
4.5	0	0	0	0	0	0	0	0	1	0	0	0	0	0	0	9	-0.693147181
4.5	0	0	0	0	0	0	0	0	0	1	0	0	0	0	0	9	-0.693147181
4.5	0	0	0	0	0	0	0	0	0	0	1	0	0	0	0	9	-1.386294361
4.5	0	0	0	0	0	0	0	0	0	0	0	1	0	0	0	9	-0.693147181
4.5	0	0	0	0	0	0	0	0	0	0	0	0	1	0	0	9	-0.693147181
4.5	0	0	0	0	0	0	0	0	0	0	0	0	0	1	0	9	-1.386294361
;
run;

/*******************************************************************
Using the unscaled A=4.5 prior
********************************************************************/;
proc logistic data=prior1;		 	 
	title "Logistic regression to get back priors";
	title2 "A=4.5, non-zero mean";
	model death/M1 = const nonwhite--malpres/noint offset=H clodds=both;
	ods output cloddspl=prior1ors;
	run;
/*NOTE, the prior limits are way off because A<4. 
	To get correct limits use the  inverse F cumulative distribution function 
	to find the p = 0.025 and 0.975 percentiles of the F(9,9) distribution:	*/
data prior1ors;
	set prior1ors;
	lowercl=finv(0.025,9,9)*OddsRatioEst;	
	uppercl=finv(0.975,9,9)*OddsRatioEst;
	run; 
data augdata1;								 
	set fm0 prior1;
	run;
proc logistic data=augdata1;	
	title "Logistic regression to obtain posterior estimates using data augmentation";
	title2 "A=4.5, non-zero mean";
	model death/M1 = Const nonwhite--malpres/noint offset=H clodds=both;
	ods output cloddswald=postaug1w cloddspl=postaug1pl;
	run;

/*******************************************************************
Zero-centred means, A=4.5, H=0
	no need to change H, simply don't specify an offset
	if there are offsets in the data, then you will need to make H=0
********************************************************************/;
proc logistic data=augdata1;	
	title "Logistic regression to obtain posterior estimates using data augmentation";
	title2 "A=4.5, zero-centred mean";
	model death/M1 = Const nonwhite--malpres/noint clodds=both;
	ods output cloddswald=postaug2w cloddspl=postaug2pl;
	run;

/*******************************************************************
Weak prior, A=1, non-zero-centred means
********************************************************************/;
data prior2;
	set prior1;
	death=1;
	M1=2;
	run;
/*Output priors*/;
proc logistic data=prior2;
	title "Logistic regression to get back priors";
	title2 "A=1, non-zero-centred";
	model death/M1 = const nonwhite--malpres/noint offset=H clodds=both;
	ods output cloddspl=prior2ors;
run;
/*NOTE, the prior limits are way off because A<4. 
	To get correct limits use the  inverse F cumulative distribution function 
	to find the p = 0.025 and 0.975 percentiles of the F(2,2) distribution:	*/
data prior2ors;
	set prior2ors;
	lowercl=finv(0.025,2,2)*OddsRatioEst;	
	uppercl=finv(0.975,2,2)*OddsRatioEst;
	run;
/*Compute posterior estimates when rescaled*/;
data augdata3;								 
	set fm0 prior2;
	run;
proc logistic data=augdata3;	
	title "Bayesian logistic regression of death on 14 regressors using data augmentation";
	title2 "A=1, non-zero-centred";
	model death/M1 = const nonwhite--malpres/noint offset=H clodds=both;
	ods output cloddswald=postaug3w cloddspl=postaug3pl;
	run;

/*******************************************************************
Weak prior, A=1, zero-centred means, H=0
********************************************************************/;
data prior4;
	set prior1;
	death=1;
	M1=2;
	H=0;
	run;
/*Compute posterior estimates when rescaled*/;
data augdata4;								 
	set fm0 prior4;
	run;
proc logistic data=augdata4;	
	title "Bayesian logistic regression of death on 14 regressors using data augmentation";
	title2 "A=1, non-zero-centred";
	model death/M1 = const nonwhite--malpres/noint clodds=both;
	ods output cloddswald=postaug4w cloddspl=postaug4pl;
	run;

/*******************************************************************
Table 3
********************************************************************/;
proc sort data=prior1ors; by effect;
proc sort data=prior2ors; by effect;
proc sort data=actualw; by effect;
proc sort data=actualpl; by effect;
proc sort data=postaug1w; by effect;
proc sort data=postaug1pl; by effect;
proc sort data=postaug2w; by effect;
proc sort data=postaug2pl; by effect;
proc sort data=postaug3w; by effect;
proc sort data=postaug3pl; by effect;
proc sort data=postaug4w; by effect;
proc sort data=postaug4pl; by effect;
data table3;
	merge 	prior1ors(rename=(OddsRatioEst=prior1OR LowerCL=prior1LL UpperCL=prior1UL)) 
			prior2ors(rename=(OddsRatioEst=prior2OR LowerCL=prior2LL UpperCL=prior2UL)) 
			actualw(rename=(OddsRatioEst=actualwOR LowerCL=actualwLL UpperCL=actualwUL)) 
			actualpl(rename=(OddsRatioEst=actualplOR LowerCL=actualplLL UpperCL=actualplUL)) 
			postaug1w(rename=(OddsRatioEst=postaug1wOR LowerCL=postaug1wLL UpperCL=postaug1wUL)) 
			postaug1pl(rename=(OddsRatioEst=postaug1plOR LowerCL=postaug1plLL UpperCL=postaug1plUL)) 
			postaug2w(rename=(OddsRatioEst=postaug2wOR LowerCL=postaug2wLL UpperCL=postaug2wUL)) 
			postaug2pl(rename=(OddsRatioEst=postaug2plOR LowerCL=postaug2plLL UpperCL=postaug2plUL))
			postaug3w(rename=(OddsRatioEst=postaug3wOR LowerCL=postaug3wLL UpperCL=postaug3wUL)) 
			postaug3pl(rename=(OddsRatioEst=postaug3plOR LowerCL=postaug3plLL UpperCL=postaug3plUL)) 
			postaug4w(rename=(OddsRatioEst=postaug4wOR LowerCL=postaug4wLL UpperCL=postaug4wUL)) 
			postaug4pl(rename=(OddsRatioEst=postaug4plOR LowerCL=postaug4plLL UpperCL=postaug4plUL))
			;
	by effect;

*decimal places are to 2 informative digits, regardless of decimal point;
*prior;
	%rounding1(prior1OR,prior1LL,prior1UL,prior1);
	%rounding1(prior2OR,prior2LL,prior2UL,prior2);
*actual;
	%rounding2(actualwOR,actualwLL,actualwUL,actualw);
	%rounding2(actualplOR,actualplLL,actualplUL,actualpl);
	actual = (trim(left(actualw))) || ' (' || trim(left(ll_actualpl)) || ',' || trim(left(ul_actualpl)) || ')';	
*posterior, data augmentation, increment, non-zero-centred median;
	%rounding2(postaug1wOR,postaug1wLL,postaug1wUL,postaug1w);
	%rounding2(postaug1plOR,postaug1plLL,postaug1plUL,postaug1pl);
	postaug1 = (trim(left(postaug1w))) || ' (' || trim(left(ll_postaug1pl)) || ',' || trim(left(ul_postaug1pl)) || ')';
*posterior, data augmentation, increment, zero-centred median;
	%rounding2(postaug2wOR,postaug2wLL,postaug2wUL,postaug2w);
	%rounding2(postaug2plOR,postaug2plLL,postaug2plUL,postaug2pl);
	postaug2 = (trim(left(postaug2w))) || ' (' || trim(left(ll_postaug2pl)) || ',' || trim(left(ul_postaug2pl)) || ')';
*posterior, data augmentation, weak, non-zero-centred median;
	%rounding2(postaug3wOR,postaug3wLL,postaug3wUL,postaug3w);
	%rounding2(postaug3plOR,postaug3plLL,postaug3plUL,postaug3pl);
	postaug3 = (trim(left(postaug3w))) || ' (' || trim(left(ll_postaug3pl)) || ',' || trim(left(ul_postaug3pl)) || ')';
*posterior, data augmentation, weak, zero-centred median;
	%rounding2(postaug4wOR,postaug4wLL,postaug4wUL,postaug4w);
	%rounding2(postaug4plOR,postaug4plLL,postaug4plUL,postaug4pl);
	postaug4 = (trim(left(postaug4w))) || ' (' || trim(left(ll_postaug4pl)) || ',' || trim(left(ul_postaug4pl)) || ')';

*set order;
	if effect = "Const" then do; regressor = "Const"; order=1; end;
	if effect = "nonwhite" then do; regressor = "Non-white"; order=2; end;
	if effect = "teenages" then do; regressor = "Early age"; order=3; end;
	if effect = "nullip" then do; regressor = "Nulliparity"; order=4; end;
	if effect = "gestage" then do; regressor = "Gestational age"; order=5; end;
	if effect = "isoimm" then do; regressor = "Isoimmunization "; order=6; end;
	if effect = "abort" then do; regressor = "Past abortion"; order=7; end;
	if effect = "hydram" then do; regressor = "Hydramnios"; order=8; end;
	if effect = "dyslab" then do; regressor = "Labour progress "; order=9; end;
	if effect = "placord" then do; regressor = "PCA"; order=10; end;
	if effect = "nomonit" then do; regressor = "No monitor"; order=11; end;
	if effect = "twint" then do; regressor = "Twin, triplet"; order=12; end;
	if effect = "ward" then do; regressor = "Public ward"; order=13; end;
	if effect = "prerupt" then do; regressor = "PROM"; order=14; end;
	if effect = "malpres" then do; regressor = "Malpresented"; order=15; end;

	label	regressor = "Regressor (Xj)a"
			prior1 = "Prior 1, A=4.5, median ORprior (95% limits)"
			prior2 = "Prior 2, A=1, median ORprior (95% limits)"
			actual = "ML estimates, A=0, (95% WL)(95% PL)"
			postaug1 = "A=4.5, non-zero-centred prior median (95% WL)(95% PL)"
			postaug2 = "A=4.5, zero-centred prior median (95% WL)(95% PL)"
			postaug3 = "A=1, non-zero-centred prior median (95% WL)(95% PL)"
			postaug4 = "A=1, zero-centred prior median (95% WL)(95% PL)"
;	run;
proc sort; by order; run;
ods rtf file="&workdir\Table3.rtf";
proc print data=table3 noobs label;
	title "Table 3. Sensitivity analysis";
	var regressor prior1 prior2 actual postaug1 postaug2 postaug3 postaug4;
	run;
ods rtf close;
ods listing;
